import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/chat_search_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/v2/chat_service.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:qq_delivery/delivery/view/widget/address_widget.dart';
import 'package:shared_preferences/shared_preferences.dart';


class ChatSearch extends StatefulWidget {
  ChatSearch({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _ChatSearchState createState() => _ChatSearchState();
}

class _ChatSearchState extends State<ChatSearch> {

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  bool _first = true;

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  ChatService _chatService = new ChatService();
  ChatSearchListResponse _chatSearchListResponse;
  List<SearchListItem> _items = [];
  int _page = 0;

  String _keyword = '';
  final _searchQuery = new TextEditingController();
  Timer _debounce;


  @override
  void initState() {
    super.initState();
    _searchQuery.addListener(_onSearchChanged);
  }
  @override
  void deactivate() {
    super.deactivate( );
  }
  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    if (_debounce?.isActive ?? false) _debounce.cancel();
    super.dispose();
  }

  _onSearchChanged() {
    setState(() { });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(seconds: 3), () {
      _keyword = _searchQuery.text.trim();
      if( (_keyword.length ==0 || _keyword.length >=3) && !_refreshController.isRefresh){
        _refreshController.requestRefresh();
      }
    });
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }

    _chatSearchListResponse = await _chatService.searchPersonOrGroup(
      "all", _keyword, _page, Constant.ROW_PER_PAGE );


    if( refresh && _items.isNotEmpty)
      _items.removeRange(0, _items.length);

    if( _chatSearchListResponse!=null && _chatSearchListResponse.items!=null){
      _chatSearchListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          appBar: AppBar(
            title: Text(widget.title, style: TextStyle(color: Colors.white)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),
            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          )
        )
    );
  }

  Widget _topWidget(BuildContext context){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, bottom: 8.0, left: 0.0, right: 0.0),
      child: Card(
        // color: Colors.white,
        elevation: 0,
        child: new Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment:CrossAxisAlignment.start,
          children: <Widget>[
            Container(
              padding: EdgeInsets.only(top: 16, left: 16, bottom: 0, right: 16),
              child: TextField(
                textAlign: TextAlign.center,
                keyboardType: TextInputType.text,
                decoration: InputDecoration(
                    hintText: tr(LocaleKeys.lblName),
                    hintStyle: TextStyle(fontSize: 14),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(16),
                      borderSide: BorderSide(
                        width: 1,
                        style: BorderStyle.none,
                      ),
                    ),
                    filled: true,
                    contentPadding: EdgeInsets.all(8),
                    fillColor: Theme.of(context).backgroundColor,
                    suffixIcon: _searchQuery.text.length>0? IconButton(
                      icon: Icon(Icons.close_sharp,
                          // color: Theme.of(context).primaryColorDark
                      ),
                      onPressed: (){
                        setState(() {
                          _searchQuery.text = '';
                          _keyword = '';
                        });
                        _refreshController.requestRefresh();
                      },
                    ): Icon(Icons.search_sharp),
                ),
                controller: _searchQuery,
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.builder(
      reverse: false,
      padding: EdgeInsets.only(left: 8, right: 8),
      itemBuilder: (c, i) => i==0? _topWidget(c) : PeopleListItem(
        post: _items[i-1],
        authorityResponse: _authorityResponse,
        chatService: _chatService,
        isEnd: i == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      itemCount: _items.length + 1,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}

class PeopleListItem extends StatefulWidget {
  PeopleListItem({Key key, @required this.post,
    @required this.authorityResponse, @required this.chatService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser}) : super(key: key);
  final SearchListItem post;
  final AuthorityResponse authorityResponse;
  ChatService chatService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _PeopleListItemState createState() => _PeopleListItemState();

}

class _PeopleListItemState extends State<PeopleListItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 0,
                  child: Wrap(
                      children: <Widget>[
                        ListTile(
                          title: Text(widget.post.name,
                            style: Theme.of(context).textTheme.headline6,
                          ),
                          leading: ClipOval(
                              child: Material(
                                elevation: 1,
                                color: Colors.transparent, // button color
                                child: InkWell(
                                  // splashColor: Colors.red, // inkwell color
                                  child: SizedBox(
                                    width: 42,
                                    height: 42,
                                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.post.avatar}', width: 42, height: 42, ),
                                  ),
                                ),
                              )
                          ),
                          trailing: Icon( Icons.check_box_outline_blank_sharp,
                            // color: Colors.blueGrey[600],
                          ),
                          onTap: (){
                            Navigator.pop( context, widget.post );
                          },
                        ),
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
