import 'dart:typed_data';
import 'package:easy_localization/easy_localization.dart';
import 'package:intl/intl.dart';
import 'package:flutter/material.dart' hide Image;
import 'package:esc_pos_printer/esc_pos_printer.dart';
import 'package:flutter/services.dart';
import 'package:ping_discover_network/ping_discover_network.dart';
import 'package:esc_pos_utils/esc_pos_utils.dart';
import 'package:image/image.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wifi/wifi.dart';
import 'package:image/image.dart' as imglib;
import 'lan_printing.dart';
import 'dart:ui' as ui;


class PrinterDiscover extends StatefulWidget {
  PrinterDiscover({Key key, this.detailResponse, this.contactItemOS,
  this.contactItemCustomer, this.sharedPrefs}) : super(key: key);
  final DeliveryDetailResponse detailResponse;
  final DeliveryContactItem contactItemOS;
  final DeliveryContactItem contactItemCustomer;
  final SharedPreferences sharedPrefs;
  @override
  _PrinterDiscoverState createState() => _PrinterDiscoverState();
}

class _PrinterDiscoverState extends State<PrinterDiscover> {
  String localIp = '';
  List<String> devices = [];
  bool isDiscovering = false;
  int found = -1;
  TextEditingController portController = TextEditingController(text: '9100');

  void discover(BuildContext ctx) async {
    setState(() {
      isDiscovering = true;
      devices.clear();
      found = -1;
    });

    String ip;
    try {
      ip = await Wifi.ip;
    } catch (e) {
      final snackBar = SnackBar(
          content: Text('WiFi is not connected', textAlign: TextAlign.center));
      Scaffold.of(ctx).showSnackBar(snackBar);
      return;
    }
    setState(() {
      localIp = ip;
    });

    final String subnet = ip.substring(0, ip.lastIndexOf('.'));
    int port = 9100;
    try {
      port = int.parse(portController.text);
    } catch (e) {
      portController.text = port.toString();
    }
    final stream = NetworkAnalyzer.discover2(subnet, port);

    stream.listen((NetworkAddress addr) {
      if (addr.exists) {
        setState(() {
          devices.add(addr.ip);
          found = devices.length;
        });
      }
    })
      ..onDone(() {
        setState(() {
          isDiscovering = false;
          found = devices.length;
        });
      })
      ..onError((dynamic e) {
        final snackBar = SnackBar(
            content: Text('Unexpected exception', textAlign: TextAlign.center));
        Scaffold.of(ctx).showSnackBar(snackBar);
      });
  }

  void printNow(String printerIp, BuildContext ctx) async {
    const PaperSize paper = PaperSize.mm80;
    final profile = await CapabilityProfile.load();
    final printer = NetworkPrinter(paper, profile);
    final PosPrintResult res = await printer.connect( printerIp, port: 9100 );

    if (res == PosPrintResult.success) {
      imglib.Image bodyImage = await _renderedToImage( widget.detailResponse,
          widget.contactItemOS, widget.contactItemCustomer, widget.sharedPrefs,  context );

      LanPrinting lanPrinting = new LanPrinting();
      await lanPrinting.printSlip(printer, widget.detailResponse, widget.contactItemOS,
          widget.contactItemCustomer, widget.sharedPrefs, printerIp, ctx, bodyImage);
      printer.disconnect();
    }

    final snackBar =
    SnackBar(content: Text(res.msg, textAlign: TextAlign.center));
    Scaffold.of(ctx).showSnackBar(snackBar);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Discover Network Printers'),
      ),
      body: Builder(
        builder: (BuildContext context) {
          return Container(
            padding: EdgeInsets.symmetric(horizontal: 10, vertical: 20),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                TextField(
                  controller: portController,
                  keyboardType: TextInputType.number,
                  decoration: InputDecoration(
                    labelText: tr(LocaleKeys.lblPrinterPort),
                    hintText: 'Printer port',
                  ),
                ),
                SizedBox(height: 10),
                Text('${tr(LocaleKeys.lblPrinterLocalIPAddress)}$localIp', style: TextStyle(fontSize: 16)),
                SizedBox(height: 15),
                RaisedButton(
                    child: Text(
                        '${isDiscovering ? '${tr(LocaleKeys.lblPrinterDiscovering)}' : '${tr(LocaleKeys.lblPrinterFindPrinter)}'}',
                      style: TextStyle(color: Colors.white),),
                    onPressed: isDiscovering ? null : () => discover(context)),
                SizedBox(height: 15),
                found >= 0
                    ? Text('Found: $found printer(s)',
                    style: TextStyle(fontSize: 16))
                    : Container(),
                Expanded(
                  child: ListView.builder(
                    itemCount: devices.length,
                    itemBuilder: (BuildContext context, int index) {
                      return InkWell(
                        onTap: () => printNow(devices[index], context),
                        child: Column(
                          children: <Widget>[
                            Container(
                              height: 60,
                              padding: EdgeInsets.only(left: 10),
                              alignment: Alignment.centerLeft,
                              child: Row(
                                children: <Widget>[
                                  Icon(Icons.print),
                                  SizedBox(width: 10),
                                  Expanded(
                                    child: Column(
                                      crossAxisAlignment:
                                      CrossAxisAlignment.start,
                                      mainAxisAlignment:
                                      MainAxisAlignment.center,
                                      children: <Widget>[
                                        Text(
                                          '${devices[index]}:${portController.text}',
                                          style: TextStyle(fontSize: 16),
                                        ),
                                        Text(
                                          '${tr(LocaleKeys.lblPrinterPressToPrint)}',
                                          style: TextStyle(
                                              color: Colors.grey[700]),
                                        ),
                                      ],
                                    ),
                                  ),
                                  Icon(Icons.chevron_right),
                                ],
                              ),
                            ),
                            Divider(),
                          ],
                        ),
                      );
                    },
                  ),
                )
              ],
            ),
          );
        },
      ),
    );
  }

  Future<imglib.Image> _renderedToImage(DeliveryDetailResponse detailResponse, DeliveryContactItem contactItemOS,
      DeliveryContactItem contactItemCustomer, SharedPreferences sharedPrefs,  BuildContext context ) async{
    int PRINT_AREA_WIDTH = context.size.width.floor();
    int PRINT_AREA_HEIGHT = 250;
    double TOP_MARGIN = 10;
    double LEFT_MARGIN = 0;
    double FONT_SIZE = 20;
    double TEXT_LINE_WIDTH = FONT_SIZE * 2;

    ui.PictureRecorder recorder = ui.PictureRecorder();
    Canvas canvas = Canvas(recorder);
    canvas.drawColor(Colors.white, ui.BlendMode.color);

    final textStyle = TextStyle(color: Colors.black, fontSize: FONT_SIZE);

// Customer info
    // text line 1
    final textSpan = TextSpan(text: 'ပစ္စည်းပို့ရန်', style: textStyle,);
    final textPainter = TextPainter(text: textSpan, textDirection: ui.TextDirection.ltr, textAlign: TextAlign.start);
    textPainter.layout(minWidth: 0, maxWidth: context.size.width);
    final offset = Offset( LEFT_MARGIN, TOP_MARGIN + (TEXT_LINE_WIDTH*0) );//x,y
    textPainter.paint(canvas, offset);

    // text line 2
    final textSpan2 = TextSpan(text: 'အမည်။ ${contactItemCustomer.name}', style: textStyle,);
    final textPainter2 = TextPainter(text: textSpan2, textDirection: ui.TextDirection.ltr);
    textPainter2.layout(minWidth: 0, maxWidth: context.size.width);
    final offset2 = Offset( LEFT_MARGIN, TOP_MARGIN + (TEXT_LINE_WIDTH*1) );//x,y
    textPainter2.paint(canvas, offset2);

    // text line 3
    final textSpan3 = TextSpan(text: 'ဖုန်းနံပါတ်။ ${contactItemCustomer.primaryPhone}${contactItemCustomer.otherPhones!=null? ","+contactItemCustomer.otherPhones:""}', style: textStyle,);
    final textPainter3 = TextPainter(text: textSpan3, textDirection: ui.TextDirection.ltr);
    textPainter3.layout(minWidth: 0, maxWidth: context.size.width);
    final offset3 = Offset( LEFT_MARGIN, TOP_MARGIN + (TEXT_LINE_WIDTH*2) );//x,y
    textPainter3.paint(canvas, offset3);

    // text line 4
    final textSpan4 = TextSpan(text: "လိပ်စာ။ ${contactItemCustomer.address + ', \n' + contactItemCustomer.street + ", " + contactItemCustomer.township}",
      style: textStyle,);
    final textPainter4 = TextPainter(text: textSpan4, textDirection: ui.TextDirection.ltr);
    textPainter4.layout(minWidth: 0, maxWidth: context.size.width);
    final offset4 = Offset( LEFT_MARGIN, TOP_MARGIN + (TEXT_LINE_WIDTH*3) );//x,y
    textPainter4.paint(canvas, offset4);

    // text line 5
    final textSpan5 = TextSpan(text: '${tr(LocaleKeys.lblCashToCollectInPrint)} ${new NumberFormat("#,###").format(detailResponse.totalCharges)} Ks', style: textStyle,);
    final textPainter5 = TextPainter(text: textSpan5, textDirection: ui.TextDirection.ltr);
    textPainter5.layout(minWidth: 0, maxWidth: context.size.width);
    final offset5 = Offset( LEFT_MARGIN, TOP_MARGIN + (TEXT_LINE_WIDTH*5) );//x,y
    textPainter5.paint(canvas, offset5);

    // creating image
    ui.Image imagePicture = await recorder.endRecording()
        .toImage( PRINT_AREA_WIDTH, PRINT_AREA_HEIGHT );// print area width, height
    var pngBytes =  await imagePicture.toByteData(format: ui.ImageByteFormat.png);
    imglib.Image imageToPrint= imglib.decodeImage( pngBytes.buffer.asUint8List() );
    return imageToPrint;
  }

}