import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_html_to_pdf/flutter_html_to_pdf.dart';
import 'package:flutter_uploader/flutter_uploader.dart';
import 'package:intl/intl.dart';
import 'package:flutter/material.dart' hide Image;
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/active_printer_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/message_item.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/remote_printer_service.dart';
import 'package:qq_delivery/delivery/view/chat/chat_conversation_list.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wifi/wifi.dart';
import 'package:image/image.dart' as imglib;
import 'lan_printing.dart';
import 'dart:ui' as ui;


class RemotePrinterDiscover extends StatefulWidget {
  RemotePrinterDiscover({Key key, this.detailResponse, this.contactItemOS,
  this.contactItemCustomer, this.sharedPrefs}) : super(key: key);
  final DeliveryDetailResponse detailResponse;
  final DeliveryContactItem contactItemOS;
  final DeliveryContactItem contactItemCustomer;
  final SharedPreferences sharedPrefs;
  @override
  _PrinterDiscoverState createState() => _PrinterDiscoverState();
}

class _PrinterDiscoverState extends State<RemotePrinterDiscover> {

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  ActivePrinterResponse _activePrinterResponse;
  RemotePrinterService _printerService = new RemotePrinterService();
  List<PrinterItem> _items = [];
  int _page = 0;
  bool _first = true;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();

  FlutterUploader uploader = FlutterUploader();
  StreamSubscription _progressSubscription;
  StreamSubscription _resultSubscription;
  Map<String, UploadItem> _tasks = {};
  String _mediaFileGuid;
  bool _recording = false;
  bool isStopped = false;
  int _totalRecords = 0;
  String _printMessage = "";

  @override
  void initState() {
    super.initState();

    ////// START OF FILE UPLOAD /////////////////////////////////
    _progressSubscription = uploader.progress.listen((progress) {
      final task = _tasks[progress.tag];
      print("progress: ${progress.progress} , tag: ${progress.tag}");
      if (task == null) return;
      if (task.isCompleted()) return;
      setState(() {
        _tasks[progress.tag] =
            task.copyWith(progress: progress.progress, status: progress.status);
      });
    });

    _resultSubscription = uploader.result.listen((result) {
      // print("id: ${result.taskId}, status: ${result.status}, response: ${result.response}, statusCode: ${result.statusCode}, tag: ${result.tag}, headers: ${result.headers}");
      Map<String, dynamic> resultData = jsonDecode(result.response);
      if( resultData['success']){
        _mediaFileGuid = resultData['data'];
        _sendMediaMessage( _tasks[result.tag] );
      }
      final task = _tasks[result.tag];
      if (task == null) return;

      setState(() {
        _tasks[result.tag] = task.copyWith(status: result.status);
      });

      String printerGuid = task.tag;
      _printerService.queueCreate(printerGuid, _mediaFileGuid);
      setState(() {
        _processing = false;
      });
    }, onError: (ex, stacktrace) {
      print("exception: $ex");
      print("stacktrace: $stacktrace" ?? "no stacktrace");
      final exp = ex as UploadException;
      final task = _tasks[exp.tag];
      if (task == null) return;

      setState(() {
        _tasks[exp.tag] = task.copyWith(status: exp.status);
        _processing = false;
      });
    });
    // END OF FILE UPLOAD //////////////////////
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = widget.sharedPrefs;
      _first = false;
    }
    setState(() {});

    _activePrinterResponse = await _printerService.list();
    if( _activePrinterResponse !=null && _activePrinterResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _activePrinterResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(tr(LocaleKeys.lblEmployeeClearance), style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }


  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => PrinterListItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: (guid) async{
          _generatePDFDocument(context, guid);
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  Future<void> _generatePDFDocument( BuildContext context, String guid ) async {

      setState(() {
        _processing = true;
      });
      var htmlContent = await _deliveryContentService.wayBillToPrint(
          widget.detailResponse.trackingId, "80x60");
      Directory appDocDir = await getApplicationDocumentsDirectory();
      var targetPath = appDocDir.path;
      var targetFileName = "${widget.detailResponse.trackingId}";

      var generatedPdfFile = await FlutterHtmlToPdf.convertFromHtmlContent(
          htmlContent, targetPath, targetFileName);
      String generatedPdfFilePath = generatedPdfFile.path;
      await _getFile(generatedPdfFilePath, guid);
  }

  Future<void> _getFile(String generatedPdfFilePath, String guid) async{
    _mediaFileGuid = null;
    try{
      if(generatedPdfFilePath != null) {
        File file = File( generatedPdfFilePath );
        // File file = File(result.files.single.path);
        final String filename = basename(file.path);
        final String savedDir = dirname(file.path);
        final tag = "file upload ${_tasks.length + 1}";
        final taskId = await uploader.enqueue(
            url: '${getBaseURL(widget.sharedPrefs)}/v3/files/upload',
            files: [FileItem(filename: filename, savedDir: savedDir, fieldname:"file")],
            method: UploadMethod.POST,
            // headers: {"apikey": "api_123456", "userkey": "userkey_123456"},
            // data: {"name": "john"},
            showNotification: false,
            tag: tag);

        setState(() {
          _tasks.putIfAbsent(
              tag,
                  () => UploadItem(
                id: taskId,
                tag: guid,
                type: "File",
                status: UploadTaskStatus.enqueued,
                filePath: generatedPdfFilePath,
              ));
        });
      }
    }catch(e){}
  }

  Future cancelUpload(String id) async {
    await uploader.cancel(taskId: id);
  }
  void _sendMediaMessage(UploadItem task) async{
    try{
        File recordFile = File(task.filePath);
        recordFile.deleteSync(recursive: false);
    }catch(e){}
  }
}


class PrinterListItem extends StatefulWidget {

  PrinterListItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final PrinterItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _PrinterListItemState createState() => _PrinterListItemState();

}

class _PrinterListItemState extends State<PrinterListItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        ListTile(
                          title: Text(
                            widget.post.name,
                            style: Theme.of(context).textTheme.headline6,
                          ),
                          subtitle: Text(widget.post.available?'Available' : 'Offline',),
                          trailing: widget.post.available?Icon(Icons.print) : Icon(Icons.print_disabled_sharp),
                          onTap: !widget.post.available? null : (){
                            widget.onChange(widget.post.guid);
                          },
                        ),
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }
}
