
import 'dart:async';
import 'dart:io';
import 'package:camera/camera.dart';
import 'package:device_info/device_info.dart';
import '/hrm/config/constant.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:path_provider/path_provider.dart';
import 'package:visibility_detector/visibility_detector.dart';


// typedef HandleDetection<T> = Future<T> Function(FirebaseVisionImage image);
typedef Widget ErrorWidgetBuilder(BuildContext context, CameraError error);

enum CameraError {
  unknown,
  cantInitializeCamera,
  androidVersionNotSupported,
  noCameraAvailable,
}

enum _CameraState {
  loading,
  error,
  ready,
}

class CameraMlVision<T> extends StatefulWidget {
  final WidgetBuilder loadingBuilder;
  final ErrorWidgetBuilder errorBuilder;
  final WidgetBuilder overlayBuilder;
  final CameraLensDirection cameraLensDirection;
  final ResolutionPreset resolution;
  final Function onDispose;
  final WidgetBuilder overlayBuilder1;


  CameraMlVision({
    Key key,
    this.loadingBuilder,
    this.errorBuilder,
    this.overlayBuilder,
    this.cameraLensDirection = CameraLensDirection.back,
    this.resolution,
    this.onDispose,
    this.overlayBuilder1
  }) : super(key: key);

  @override
  CameraMlVisionState createState() => CameraMlVisionState<T>();
}

class CameraMlVisionState<T> extends State<CameraMlVision<T>> {
  String _lastImage;
  Key _visibilityKey = UniqueKey();
  CameraController _cameraController;
  Future<void> _initializeControllerFuture;
  // ImageRotation _rotation;
  _CameraState _cameraMlVisionState = _CameraState.loading;
  CameraError _cameraError = CameraError.unknown;
  bool _alreadyCheckingImage = false;
  bool _isDeactivate = false;

  @override
  void initState() {
    super.initState();
    _initialize();
  }


  Future<String> takePhoto( ) async{
    String path;
    if (!mounted) {
      return path;
    }

//    try {

      if (_cameraController.value.isTakingPicture) {
        if( Constant.ENABLE_LOGS)
          print('camera picture taking'); return path;
      }
      await _initializeControllerFuture;
      XFile xFile = await _cameraController.takePicture( );
      path = xFile.path;
      return path;

//    }catch (e) {
//      print('Error from Flutter Camera takePhoto()${e}');
//    }
    return path;
  }

  Future<void> stop() async {
    if (_cameraController != null) {
      if (_lastImage != null && File(_lastImage).existsSync()) {
        await File(_lastImage).delete();
      }

      Directory tempDir = await getTemporaryDirectory();
      _lastImage = '${tempDir.path}/${DateTime.now().millisecondsSinceEpoch}';
      try {
        XFile xFile = await _cameraController.takePicture();
        _lastImage = xFile.path;
      } on PlatformException catch (e) {
        debugPrint('$e');
      }

      _stop();
    }
  }

  void _stop() {
    Future.microtask(() async {
      if (_cameraController?.value?.isStreamingImages == true && mounted) {
        await _cameraController.stopImageStream();
      }
    });
  }

  void start() {
    if (_cameraController != null) {
      _start();
    }
  }

  void _start() async{
    await _initializeControllerFuture;
    _cameraController.startImageStream(_processImage);
  }

  CameraValue get cameraValue => _cameraController?.value;
  // ImageRotation get imageRotation => _rotation;

  Future<XFile> Function() get takePicture =>
      _cameraController.takePicture;


  Future<void> _initialize() async {
    if (Platform.isAndroid) {
      final deviceInfo = DeviceInfoPlugin();
      final androidInfo = await deviceInfo.androidInfo;
      if (androidInfo.version.sdkInt < 21) {
        debugPrint('Camera plugin doesn\'t support android under version 21');
        if (mounted) {
          setState(() {
            _cameraMlVisionState = _CameraState.error;
            _cameraError = CameraError.androidVersionNotSupported;
          });
        }
        return;
      }
    }

    CameraDescription description =
    await _getCamera(widget.cameraLensDirection);
    if (description == null) {
      _cameraMlVisionState = _CameraState.error;
      _cameraError = CameraError.noCameraAvailable;

      return;
    }
    _cameraController = CameraController(
      description,
      widget.resolution ??
          ResolutionPreset
              .medium, // As the doc says, better to set low when streaming images to avoid drop frames on older devices
      enableAudio: false,
      imageFormatGroup: ImageFormatGroup.yuv420,
    );
    if (!mounted) {
      return;
    }

    try {
      //await _cameraController.initialize();
      _initializeControllerFuture = _cameraController.initialize();
    } catch (ex, stack) {
      debugPrint('Can\'t initialize camera');
      debugPrint('$ex, $stack');
      if (mounted) {
        setState(() {
          _cameraMlVisionState = _CameraState.error;
          _cameraError = CameraError.cantInitializeCamera;
        });
      }
      return;
    }

    if (!mounted) {
      return;
    }

    setState(() {
      _cameraMlVisionState = _CameraState.ready;
    });
    // _rotation = _rotationIntToImageRotation(description.sensorOrientation,);

    //FIXME hacky technique to avoid having black screen on some android devices
    if (Platform.isAndroid) {
      await Future.delayed(Duration(milliseconds: 200));
    }
    //start();
  }

  Future<CameraDescription> _getCamera(CameraLensDirection dir) async {
    return await availableCameras().then(
          (cameras) => cameras.firstWhere(
            (camera) => camera.lensDirection == dir,
        orElse: () => cameras.isNotEmpty ? cameras.first : null,
      ),
    );
  }

  @override
  void dispose() {
    if (widget.onDispose != null) {
      widget.onDispose();
    }
    if (_lastImage != null && File(_lastImage).existsSync()) {
      File(_lastImage).delete();
    }
    if (_cameraController != null) {
      _cameraController.dispose();
    }
    _cameraController = null;
    super.dispose();
  }


  @override
  Widget build(BuildContext context){

    if (_cameraMlVisionState == _CameraState.loading ) {
      return widget.loadingBuilder == null
          ? Center(child: CircularProgressIndicator())
          : widget.loadingBuilder(context);
    }
    if (_cameraMlVisionState == _CameraState.error) {
      return widget.errorBuilder == null
          ? Center(child: Text('$_cameraMlVisionState $_cameraError'))
          : widget.errorBuilder(context, _cameraError);
    }
    //print('Ratio=${_cameraController.value.previewSize==null}');

    Widget cameraPreview =_cameraController.value.previewSize!=null ? AspectRatio(
      aspectRatio: _cameraController.value.aspectRatio,
      child: CameraPreview( _cameraController, )
    ) : CameraPreview( _cameraController, );

    // cameraPreview = CameraPreview( _cameraController, );
    if (widget.overlayBuilder != null && _cameraController.value.previewSize != null) {
      cameraPreview = Stack(
        fit: StackFit.passthrough,
        children: [
          cameraPreview,
          // (_cameraController.value.isInitialized ?? false)
          //     ? AspectRatio(
          //   aspectRatio: _isLandscape()
          //       ? _cameraController.value.aspectRatio
          //       : (1 / _cameraController.value.aspectRatio),
          //   child: widget.overlayBuilder(context),
          // )
          //     : Container(),
          // (_cameraController.value.isInitialized ?? false)
          //     ? AspectRatio(
          //   aspectRatio: _isLandscape()
          //       ? _cameraController.value.aspectRatio
          //       : (1 / _cameraController.value.aspectRatio),
          //   child: widget.overlayBuilder1(context),
          // )
          //     : Container(),
          widget.overlayBuilder(context),
        widget.overlayBuilder1(context),
        ],
      );
    }

    return FutureBuilder<void>(

        future: _initializeControllerFuture,
        builder: (context, snapshot) {

          return VisibilityDetector(
            child: cameraPreview,
            // FittedBox(
            //   alignment: Alignment.center,
            //   fit: BoxFit.cover,
            //   child: _cameraController.value.previewSize != null ? SizedBox(
            //     width: _cameraController.value.previewSize.height *
            //         _cameraController.value.aspectRatio,
            //     height: _cameraController.value.previewSize.height,
            //     child: cameraPreview,
            //   ) : cameraPreview,
            // ),
            onVisibilityChanged: (VisibilityInfo info) {
              if (info.visibleFraction == 0) {
                //invisible stop the streaming
                _isDeactivate = true;
                _stop();
              } else if (_isDeactivate) {
                //visible restart streaming if needed
                _isDeactivate = false;
                _start();
              }
            },
            key: _visibilityKey,
          );

        },
    );
  }

  DeviceOrientation _getApplicableOrientation() {
    return (_cameraController.value.isRecordingVideo ?? false)
        ? _cameraController.value.recordingOrientation
        : (_cameraController.value.lockedCaptureOrientation ??
        _cameraController.value.deviceOrientation);
  }

  bool _isLandscape() {
    return [DeviceOrientation.landscapeLeft, DeviceOrientation.landscapeRight]
        .contains(_getApplicableOrientation());
  }

  void _processImage(CameraImage cameraImage) async {

    if (!_alreadyCheckingImage && mounted) {
      _alreadyCheckingImage = true;
/*
      try {
        final T results =
        await _detect<T>(cameraImage, widget.detector, _rotation);
        widget.onResult(results, cameraImage);

      } catch (ex, stack) {
        debugPrint('$ex, $stack');
      }
*/
      _alreadyCheckingImage = false;
    }

  }

}
